@@ -1,4 +1,4 @@
-# $Id: Lookahead.pm,v 1.4 2013/07/23 23:27:33 Paulo Exp $
+# $Id: Lookahead.pm,v 1.5 2013/07/26 23:25:07 Paulo Exp $
 
 package Iterator::Simple::Lookahead;
 
@@ -17,11 +17,9 @@ use strict;
 use warnings;
 
 use Carp;
-use Iterator::Simple qw( is_iterator iterator );
-use List::AllUtils qw( first_index );
-use base 'Iterator::Simple::Iterator';
+use Iterator::Simple qw( is_iterator );
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 #------------------------------------------------------------------------------
 
@@ -75,17 +73,17 @@ L<Iterator::Simple|Iterator::Simple>.
 =cut
 
 #------------------------------------------------------------------------------
-# Object contains list of computed values and iterator to compute next
+use base 'Iterator::Simple::Iterator';
 use Class::XSAccessor {
 	accessors 		=> [
-		'_queue',		# list of computed values
-		'_iter',		# iterator to compute next
+		'_look_ahead',		# list of computed values
+		'_iterators',		# list of iterators
 	],
 };
 
 sub new {
 	my($class, @items) = @_;
-	my $self = bless {_queue=>[]}, $class;
+	my $self = bless { _look_ahead => [], _iterators => [] }, $class;
 	$self->unget(@items) if @items;
 	return $self;
 }
@@ -120,22 +118,34 @@ sub peek {
 	my($self, $n) = @_;
 	$n ||= 0; croak("negative index $n") if $n < 0;
 	
-	my $queue = $self->_queue;
-	my $iter  = $self->_iter;
+	while (1) {
+		# return element if already computed
+		return $self->_look_ahead->[$n] if $n < @{$self->_look_ahead};
 	
-	return if ! $iter && $n >= @$queue;		# no more items
-	
-	while ( $n >= @$queue ) {
-		my $value = $iter->();
-		if (! defined $value) {				# iterator exhausted
-			$self->_iter( undef );
-			return;
+		# empty list of iterators -> end of input
+		return unless @{$self->_iterators};
+		
+		# get first iterator
+		my $iter = $self->_iterators->[0];
+		if ( ! defined $iter ) {
+			shift @{$self->_iterators};			# skip undefined values
+		}
+		elsif ( _is_iter($iter) ) {
+			my $value = $iter->();
+			if ( defined($value) ) {
+				# allow an iterator to get another
+				unshift @{$self->_iterators}, $value;
+			}
+			else {
+				shift @{$self->_iterators};		# exhausted
+			}
+		}
+		else {
+			push @{$self->_look_ahead}, $iter;	# not iterator
+			shift @{$self->_iterators};
 		}
-		push @$queue, $value;
 	}
-	
-	return $queue->[$n];
-}
+}	
 
 #------------------------------------------------------------------------------
 
@@ -152,7 +162,7 @@ Returns C<undef> if the stream is empty
 sub next {
 	my($self) = @_;
 	$self->peek;	# compute head element
-	return shift @{$self->_queue};
+	return shift @{$self->_look_ahead};
 }
 
 #------------------------------------------------------------------------------
@@ -167,7 +177,7 @@ before the current call, e.g. calling from the iterator:
 
   $stream->unget(1..3); return 4;
 
-will result in the values 1,2,3,4 being returned from the stream.
+will result in the values 4,1,2,3 being returned from the stream.
 
 =cut
 
@@ -175,46 +185,8 @@ will result in the values 1,2,3,4 being returned from the stream.
 
 sub unget {
 	my($self, @items) = @_;
-	
-	# remove undefined values
-	@items = grep { defined } @items;
-	
-	# search for the first iterator in the given input
-	my $first_iterator = first_index { _is_iter($_) } @items;
-	if ($first_iterator < 0) {
-		# only data
-		unshift @{$self->_queue}, @items;
-	}
-	else {
-		# keep data up to first iterator in queue, all the rest, including exiting
-		# data in the queue is moved to the iterator
-		push @items, @{$self->_queue};
-		@{$self->_queue} = splice(@items, 0, $first_iterator);
-		
-		# chain current iterator through @items
-		push @items, $self->_iter;
-		
-		# create iterator to go through @items
-		$self->_iter( iterator {
-			while (1) {
-				return unless @items;				# empty list
-				my $item = shift @items;
-				next unless defined $item;
-				
-				if ( _is_iter($item) ) {
-					my $value = $item->();
-					if ( defined $value ) {
-						# insert value to allow an iterator to get another
-						# insert the iterator so that it is called again
-						unshift @items, $value, $item;
-					}
-				}
-				else {
-					return $item;
-				}
-			}
-		} );
-	}
+	unshift @{$self->_iterators}, @items, @{$self->_look_ahead};
+	@{$self->_look_ahead} = ();
 }
 
 #------------------------------------------------------------------------------
